@@ -1,4 +1,4 @@
-# $Id: crt.py 1219 2015-01-14 16:15:27Z mwall $
+# $Id: crt.py 1226 2015-01-25 05:43:36Z mwall $
 # Copyright 2013 Matthew Wall
 """Emit loop data to file in Cumulus realtime format.
    http://wiki.sandaysoft.com/a/Realtime.txt
@@ -33,8 +33,6 @@ The cumulus 'specification' for realtime.txt is ambiguous:
 """
 
 # FIXME: is_sunny is ill-defined.  what radiation threshold indicates true?
-# FIXME: ensure day/month/hour/minute/second is prefixed with leading zero
-# FIXME: max_rad is not yet implemented
 # FIXME: is rainYear past 365 days or since 1 jan?
 # FIXME: is rainMonth past 30 days or since 1st of month?
 
@@ -43,12 +41,13 @@ import time
 import syslog
 
 import weewx
+import weewx.almanac
 import weewx.wxformulas
 import weeutil.weeutil
 import weeutil.Sun
 from weewx.engine import StdService
 
-VERSION = "0.10"
+VERSION = "0.12"
 
 if weewx.__version__ < "3":
     raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
@@ -66,6 +65,8 @@ def loginf(msg):
 def logerr(msg):
     logmsg(syslog.LOG_ERR, msg)
 
+KM_PER_MILE = 1.60934
+
 COMPASS_POINTS = ['N','NNE','NE','ENE','E','ESE','SE','SSE',
                   'S','SSW','SW','WSW','W','WNW','NW','NNW','N']
 
@@ -73,6 +74,7 @@ COMPASS_POINTS = ['N','NNE','NE','ENE','E','ESE','SE','SSE',
 UNITS_WIND = {'mile_per_hour': 'mph',
               'meter_per_second': 'm/s',
               'kilometer_per_hour': 'km/h',
+              'km_per_hour': 'km/h',
               'knot': 'kts'}
 UNITS_TEMP = {'degree_C': 'C',
               'degree_F': 'F'}
@@ -82,6 +84,10 @@ UNITS_PRES = {'inHg': 'in',
 UNITS_RAIN = {'in': 'in',
               'mm': 'mm'}
 
+def _convert(from_v, from_units, to_units, group):
+    vt = (from_v, from_units, group)
+    return weewx.units.convert(vt, to_units)[0]
+
 def degreeToCompass(x):
     if x is None:
         return None
@@ -118,6 +124,10 @@ def knotToBeaufort(x):
         return 11 # violent storm
     return 12 # hurricane
 
+def get_db_units(dbm):
+    val = dbm.getSql("SELECT usUnits FROM %s LIMIT 1" % dbm.table_name)
+    return val[0] if val is not None else None
+
 def calcAvgWindSpeed(dbm, ts, interval=600):
     sts = ts - interval
     val = dbm.getSql("SELECT AVG(windSpeed) FROM %s "
@@ -147,27 +157,35 @@ def calc10MinAvgWindDir(dbm, ts):
     return val[0] if val is not None else None
 
 # calculate wind run since midnight
-def calcWindRun(dbm, ts):
+def calcWindRun(dbm, ts, db_us):
     run = 0
     sod_ts = weeutil.weeutil.startOfDay(ts)
     for row in dbm.genSql("SELECT `interval`,windSpeed FROM %s "
                           "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                           (sod_ts, ts)):
         if row[1] is not None:
-            run += row[1] * (row[0] / 60)
+            inc = row[1] * row[0]
+            if db_us == weewx.METRICWX:
+                inc *= 60.0
+            else:
+                inc /= 60.0
+            run += inc
     return run
 
-# calculate trend over past n hours, default to 3 hour window
-def calcTrend(label, new_val, dbm, ts, n=3):
-    if new_val is None:
-        return None
+# get trend over past n hours, default to 3 hour window
+def getTrend(label, dbm, ts, n=3):
     lastts = ts - n * 3600
     old_val = dbm.getSql("SELECT %s FROM %s "
                          "WHERE dateTime>? AND dateTime<=?" %
                          (label, dbm.table_name), (lastts, ts))
     if old_val is None or old_val[0] is None:
         return None
-    return new_val - old_val[0]
+    return old_val[0]
+
+def calcTrend(newval, oldval):
+    if newval is None or oldval is None:
+        return None
+    return newval - oldval
 
 def calcRainHour(dbm, ts):
     sts = ts - 3600
@@ -198,6 +216,13 @@ def calcRainYesterday(dbm, ts):
                      (sts, ts))
     return val[0] if val is not None else None
 
+def calcDayRain(dbm, ts):
+    sts = weeutil.weeutil.startOfDay(ts)
+    val = dbm.getSql("SELECT SUM(rain) FROM %s "
+                     "WHERE dateTime>=? AND dateTime<=?" % dbm.table_name, 
+                     (sts, ts))
+    return val[0] if val is not None else None
+
 def calcETToday(dbm, ts):
     sts = weeutil.weeutil.startOfDay(ts)
     val = dbm.getSql("SELECT SUM(ET) FROM %s "
@@ -257,6 +282,25 @@ def calcCloudBase(outTemp_F, dewpoint_F, altitude_foot):
         return None
     return int((outTemp_F - dewpoint_F) / 4.4 * 1000) + altitude_foot
 
+def calcMaxRad(alm, altitude_m):
+    try:
+        rs_el = alm.sun.alt
+        rs_R = alm.sun.earth_distance
+        rs_atc = 0.8
+        rs_z = altitude_m
+        rs_r0 = 1367
+        sinal = math.sin(rs_el)
+        if sinal < 0:
+            return 0
+        al = math.asin(sinal)
+        a0 = al * 57.2957795
+        rm = math.pow((288-0.0065*rs_z)/288,5.256)/(sinal+0.15*math.pow(a0+3.885,-1.253))
+        rs_toa = rs_r0 * sinal / (rs_R * rs_R)
+        return rs_toa * math.pow(rs_atc, rm)
+    except Exception, e:
+        logdbg("cannot calculate max solar radiation: %s" % e)
+        return 0
+
 # this depends on the weather station.  many stations have more than one.
 #
 # Vantage
@@ -307,6 +351,7 @@ class CumulusRealTime(StdService):
                                                "foot")[0]
         self.latitude = engine.stn_info.latitude_f
         self.longitude = engine.stn_info.longitude_f
+        self.db_us = None  # unit system of the database
 
         d = config_dict.get('CumulusRealTime', {})
         self.filename = d.get('filename', '/var/tmp/realtime.txt')
@@ -349,115 +394,156 @@ class CumulusRealTime(StdService):
             f.write(self.create_realtime_string(data))
             f.write("\n")
 
-    # calculate the data elements that cumulus needs that weewx does not
-    # provide directly.
+    def _cvt(self, from_v, to_us, obs, group):
+        from_units = weewx.units.getStandardUnitType(self.db_us, obs)[0]
+        to_units = weewx.units.getStandardUnitType(to_us, obs)[0]
+        return _convert(from_v, from_units, to_units, group)
+
+    # calculate the data elements that that weewx does not provide directly.
     def calculate(self, packet, dbm):
         ts = packet.get('dateTime')
-        pu = packet.get('usUnits')
-        if self.unit_system is not None and self.unit_system != pu:
+        pkt_us = packet.get('usUnits')
+        if self.unit_system is not None and self.unit_system != pkt_us:
             packet = weewx.units.to_std_system(packet, self.unit_system)
-            pu = self.unit_system
+            pkt_us = self.unit_system
+
+        # the 'from' unit system is whatever the database is using.  get it
+        # from the database once then cache it for use in conversions.
+        if self.db_us is None:
+            self.db_us = get_db_units(dbm)
+
+        p_u = weewx.units.getStandardUnitType(pkt_us, 'barometer')[0]
+        t_u = weewx.units.getStandardUnitType(pkt_us, 'outTemp')[0]
+        r_u = weewx.units.getStandardUnitType(pkt_us, 'rain')[0]
+        ws_u = weewx.units.getStandardUnitType(pkt_us, 'windSpeed')[0]
 
         data = dict(packet)
-        data['windSpeed_avg'] = calcAvgWindSpeed(dbm, ts)
+        data['windSpeed_avg'] = self._cvt(
+            calcAvgWindSpeed(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
         data['windDir_compass'] = degreeToCompass(packet.get('windDir'))
-        x = packet.get('windSpeed')
-        ut = weewx.units.getStandardUnitType(pu, 'windSpeed')
-        vt = (x, ut[0], 'group_speed')
-        v = weewx.units.convert(vt, 'knot')[0]
+        v = _convert(packet.get('windSpeed'), ws_u, 'knot', 'group_speed')
         data['windSpeed_beaufort'] = knotToBeaufort(v)
-        data['units_wind'] = UNITS_WIND.get(ut[0])
-        ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-        logerr("ut: %s pu: %s" % (ut, pu))
-        data['units_temperature'] = UNITS_TEMP.get(ut[0],ut[0])
-        ut = weewx.units.getStandardUnitType(pu, 'barometer')
-        data['units_pressure'] = UNITS_PRES.get(ut[0],ut[0])
-        ut = weewx.units.getStandardUnitType(pu, 'rain')
-        data['units_rain'] = UNITS_RAIN.get(ut[0],ut[0])
-        data['wind_run'] = calcWindRun(dbm, ts)
-        data['pressure_trend'] = calcTrend('barometer',
-                                           packet.get('barometer'), dbm, ts)
-        data['temperature_trend'] = calcTrend('outTemp',
-                                              packet.get('outTemp'), dbm, ts)
-        data['rain_month'] = calcRainMonth(dbm, ts)
-        data['rain_year'] = calcRainYear(dbm, ts)
-        data['rain_yesterday'] = calcRainYesterday(dbm, ts)
+        data['units_wind'] = UNITS_WIND.get(ws_u, ws_u)
+        data['units_temperature'] = UNITS_TEMP.get(t_u, t_u)
+        data['units_pressure'] = UNITS_PRES.get(p_u, p_u)
+        data['units_rain'] = UNITS_RAIN.get(r_u, r_u)
+        # weewx does not know distance, so we have to convert this directly
+        wr = calcWindRun(dbm, ts, self.db_us)
+        if self.db_us != pkt_us:
+            if self.db_us == weewx.US:
+                if pkt_us == weewx.METRIC:
+                    wr *= KM_PER_MILE
+                elif pkt_us == weewx.METRICWX:
+                    wr *= KM_PER_MILE * 1000
+            elif self.db_us == weewx.METRIC:
+                if pkt_us == weewx.US:
+                    wr /= KM_PER_MILE
+                elif pkt_us == weewx.METRICWX:
+                    wr *= 1000
+            elif self.db_us == weewx.METRICWX:
+                if pkt_us == weewx.US:
+                    wr /= (KM_PER_MILE * 1000)
+                elif pkt_us == weewx.METRIC:
+                    wr /= 1000
+        # wind run is mile, kilometer, or meter
+        data['wind_run'] = wr
+        p1 = packet.get('barometer')
+        p2 = getTrend('barometer', dbm, ts)
+        p2 = self._cvt(p2, pkt_us, 'barometer', 'group_pressure')
+        data['pressure_trend'] = calcTrend(p1, p2)
+        t1 = packet.get('outTemp')
+        t2 = getTrend('outTemp', dbm, ts)
+        t2 = self._cvt(t2, pkt_us, 'outTemp', 'group_temperature')
+        data['temperature_trend'] = calcTrend(t1, t2)
+        data['rain_month'] = self._cvt(
+            calcRainMonth(dbm, ts), pkt_us, 'rain', 'group_rain')
+        data['rain_year'] = self._cvt(
+            calcRainYear(dbm, ts), pkt_us, 'rain', 'group_rain')
+        data['rain_yesterday'] = self._cvt(
+            calcRainYesterday(dbm, ts), pkt_us, 'rain', 'group_rain')
+        data['dayRain'] = self._cvt(
+            calcDayRain(dbm, ts), pkt_us, 'rain', 'group_rain')
         v,t = calcMinMax('outTemp', dbm, ts, 'MAX')
-        data['outTemp_max'] = v
+        data['outTemp_max'] = self._cvt(
+            v, pkt_us, 'outTemp', 'group_temperature')
         data['outTemp_max_time'] = t
         v,t = calcMinMax('outTemp', dbm, ts, 'MIN')
-        data['outTemp_min'] = v
+        data['outTemp_min'] = self._cvt(
+            v, pkt_us, 'outTemp', 'group_temperature')
         data['outTemp_min_time'] = t
         v,t = calcMinMax('windSpeed', dbm, ts, 'MAX')
-        data['windSpeed_max'] = v
+        data['windSpeed_max'] = self._cvt(
+            v, pkt_us, 'windSpeed', 'group_speed')
         data['windSpeed_max_time'] = t
         v,t = calcMinMax('windGust', dbm, ts, 'MAX')
-        data['windGust_max'] = v
+        data['windGust_max'] = self._cvt(
+            v, pkt_us, 'windGust', 'group_speed')
         data['windGust_max_time'] = t
         v,t = calcMinMax('barometer', dbm, ts, 'MAX')
-        data['pressure_max'] = v
+        data['pressure_max'] = self._cvt(
+            v, pkt_us, 'barometer', 'group_pressure')
         data['pressure_max_time'] = t
         v,t = calcMinMax('barometer', dbm, ts, 'MIN')
-        data['pressure_min'] = v
+        data['pressure_min'] = self._cvt(
+            v, pkt_us, 'barometer', 'group_pressure')
         data['pressure_min_time'] = t
-        data['10min_high_gust'] = calc10MinHighGust(dbm, ts)
-        ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-        vt = (packet.get('outTemp'), ut[0], 'group_temperature')
-        v = weewx.units.convert(vt, 'degree_F')[0]
-        x = weewx.wxformulas.heatindexF(v, packet.get('outHumidity'))
-        vt = (x, 'degree_F', 'group_temperature')
-        v = weewx.units.convert(vt, ut[0])[0]
-        data['heatindex'] = v
-        ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-        vt = (packet.get('outTemp'), ut[0], 'group_temperature')
-        t_C = weewx.units.convert(vt, 'degree_C')[0]
-        vt = (packet.get('dewpoint'), ut[0], 'group_temperature')
-        dp_C = weewx.units.convert(vt, 'degree_C')[0]
+        data['10min_high_gust'] = self._cvt(
+            calc10MinHighGust(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
+
+        t_F = _convert(packet.get('outTemp'),
+                       t_u, 'degree_F', 'group_temperature')
+        x = weewx.wxformulas.heatindexF(t_F, packet.get('outHumidity'))
+        data['heatindex'] = _convert(x, 'degree_F', t_u, 'group_temperature')
+        t_C = _convert(packet.get('outTemp'),
+                       t_u, 'degree_C', 'group_temperature')
+        dp_C = _convert(packet.get('dewpoint'),
+                        t_u, 'degree_C', 'group_temperature')
         dp_K = dp_C + 273.15 if dp_C is not None else None
         v = calcHumidex(t_C, dp_K);
-        vt = (v, 'degree_C', 'group_temperature')
-        v = weewx.units.convert(vt, ut[0])[0]
-        data['humidex'] = v
+        data['humidex'] = _convert(v, 'degree_C', t_u, 'group_temperature')
         data['ET_today'] = calcETToday(dbm, ts)
         data['10min_avg_wind_bearing'] = calc10MinAvgWindDir(dbm, ts)
-        data['rain_hour'] = calcRainHour(dbm, ts)
+        data['rain_hour'] = self._cvt(
+            calcRainHour(dbm, ts), pkt_us, 'rain', 'group_rain')
         data['is_daylight'] = calcIsDaylight(ts, self.latitude, self.longitude)
         data['lost_sensors_contact'] = lostSensorContact(packet)
         data['avg_wind_dir'] = degreeToCompass(calcAvgWindDir(dbm, ts))
-        ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-        vt = (packet.get('outTemp'), ut[0], 'group_temperature')
-        t_F = weewx.units.convert(vt, 'degree_F')[0]
-        ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-        vt = (packet.get('dewpoint'), ut[0], 'group_temperature')
-        d_F = weewx.units.convert(vt, 'degree_F')[0]
-        cb_f = calcCloudBase(t_F, d_F, self.altitude_ft)
-        ut = weewx.units.getStandardUnitType(pu, 'altitude')
+        dp_F = _convert(packet.get('dewpoint'),
+                        t_u, 'degree_F', 'group_temperature')
+        cb_f = calcCloudBase(t_F, dp_F, self.altitude_ft)
+        ut = weewx.units.getStandardUnitType(pkt_us, 'altitude')
         vt = (cb_f, "foot", "group_altitude")
         data['cloud_base'] = weewx.units.convert(vt, ut[0])[0]
         data['units_cloud_base'] = ut[0]
         if (v < 50):
-            ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-            vt = (packet.get('outTemp'), ut[0], 'group_temperature')
-            t_F = weewx.units.convert(vt, 'degree_F')[0]
-            ut = weewx.units.getStandardUnitType(pu, 'windSpeed')
-            vt = (packet.get('windSpeed'), ut[0], 'group_speed')
-            ws_mph = weewx.units.convert(vt, ut[0])[0]
+            ws_mph = _convert(packet.get('windSpeed'),
+                              ws_u, 'mile_per_hour', 'group_speed')
             x = weewx.wxformulas.windchillF(t_F, ws_mph)
-            vt = (x, 'degree_F', 'group_temperature')
-            ut = weewx.units.getStandardUnitType(pu, 'outTemp')
-            v = weewx.units.convert(vt, ut[0])[0]
+            v = _convert(x, 'degree_F', t_u, 'group_temperature')
         else:
             v = data['heatindex']
         data['apparent_temperature'] = v
-        data['sunshine_hours'] = calcDaylightHours(ts, self.latitude, self.longitude)
-        data['max_rad'] = None
+        data['sunshine_hours'] = calcDaylightHours(
+            ts, self.latitude, self.longitude)
+        alt_m = self.altitude_ft * 0.3048
+        p_mbar = _convert(packet.get('barometer'), p_u, 'mbar', 'group_pressure')
+        alm = weewx.almanac.Almanac(ts, self.latitude, self.longitude, alt_m,
+                                    t_C, p_mbar)
+        data['max_rad'] = calcMaxRad(alm, alt_m)
         data['is_sunny'] = data['is_daylight']
         return data
 
-    def format(self, data, label):
-        value = data.get(label, 'NULL')
+    def format(self, data, label, places=None):
+        value = data.get(label)
         if value is None:
             value = 'NULL'
+        elif places is not None:
+            try:
+                v = float(value)
+                fmt = "%%.%df" % places
+                value = fmt % v
+            except ValueError:
+                pass
         return str(value)
 
     # the * indicates a field that is not part of a typical LOOP packet
@@ -466,63 +552,63 @@ class CumulusRealTime(StdService):
         fields = []
         datefmt = "%%d%s%%m%s%%y" % (self.datesep, self.datesep)
         tstr = time.strftime(datefmt, time.gmtime(data['dateTime']))
-        fields.append(tstr)                                        # 1
+        fields.append(tstr)                                           # 1
         tstr = time.strftime("%H:%M:%S", time.gmtime(data['dateTime']))
-        fields.append(tstr)                                        # 2
-        fields.append(self.format(data, 'outTemp'))                # 3
-        fields.append(self.format(data, 'outHumidity'))            # 4
-        fields.append(self.format(data, 'dewpoint'))               # 5
-        fields.append(self.format(data, 'windSpeed_avg'))          # 6  *
-        fields.append(self.format(data, 'windSpeed'))              # 7
-        fields.append(self.format(data, 'windDir'))                # 8
-        fields.append(self.format(data, 'rainRate'))               # 9
-        fields.append(self.format(data, 'dayRain'))                # 10
-        fields.append(self.format(data, 'barometer'))              # 11
-        fields.append(self.format(data, 'windDir_compass'))        # 12 *
-        fields.append(self.format(data, 'windSpeed_beaufort'))     # 13 *
-        fields.append(self.format(data, 'units_wind'))             # 14 *
-        fields.append(self.format(data, 'units_temperature'))      # 15 *
-        fields.append(self.format(data, 'units_pressure'))         # 16 *
-        fields.append(self.format(data, 'units_rain'))             # 17 *
-        fields.append(self.format(data, 'wind_run'))               # 18 *
-        fields.append(self.format(data, 'pressure_trend'))         # 19 *
-        fields.append(self.format(data, 'rain_month'))             # 20 *
-        fields.append(self.format(data, 'rain_year'))              # 21 *
-        fields.append(self.format(data, 'rain_yesterday'))         # 22 *
-        fields.append(self.format(data, 'inTemp'))                 # 23
-        fields.append(self.format(data, 'inHumidity'))             # 24
-        fields.append(self.format(data, 'windChill'))              # 25
-        fields.append(self.format(data, 'temperature_trend'))      # 26 *
-        fields.append(self.format(data, 'outTemp_max'))            # 27 *
-        fields.append(self.format(data, 'outTemp_max_time'))       # 28 *
-        fields.append(self.format(data, 'outTemp_min'))            # 29 *
-        fields.append(self.format(data, 'outTemp_min_time'))       # 30 *
-        fields.append(self.format(data, 'windSpeed_max'))          # 31 *
-        fields.append(self.format(data, 'windSpeed_max_time'))     # 32 *
-        fields.append(self.format(data, 'windGust_max'))           # 33 *
-        fields.append(self.format(data, 'windGust_max_time'))      # 34 *
-        fields.append(self.format(data, 'pressure_max'))           # 35 *
-        fields.append(self.format(data, 'pressure_max_time'))      # 36 *
-        fields.append(self.format(data, 'pressure_min'))           # 37 *
-        fields.append(self.format(data, 'pressure_min_time'))      # 38 *
-        fields.append('%s' % weewx.__version__)                    # 39
-        fields.append('0')                                         # 40
-        fields.append(self.format(data, '10min_high_gust'))        # 41 *
-        fields.append(self.format(data, 'heatindex'))              # 42 *
-        fields.append(self.format(data, 'humidex'))                # 43 *
-        fields.append(self.format(data, 'UV'))                     # 44
-        fields.append(self.format(data, 'ET_today'))               # 45 *
-        fields.append(self.format(data, 'radiation'))              # 46
-        fields.append(self.format(data, '10min_avg_wind_bearing')) # 47 *
-        fields.append(self.format(data, 'rain_hour'))              # 48 *
-        fields.append('0')                                         # 49
-        fields.append(self.format(data, 'is_daylight'))            # 50 *
-        fields.append(self.format(data, 'lost_sensors_contact'))   # 51 *
-        fields.append(self.format(data, 'avg_wind_dir'))           # 52 *
-        fields.append(self.format(data, 'cloud_base'))             # 53 *
-        fields.append(self.format(data, 'units_cloud_base'))       # 54 *
-        fields.append(self.format(data, 'apparent_temperature'))   # 55 *
-        fields.append(self.format(data, 'sunshine_hours'))         # 56 *
-        fields.append(self.format(data, 'max_rad'))                # 57 ##
-        fields.append(self.format(data, 'is_sunny'))               # 58 *
+        fields.append(tstr)                                           # 2
+        fields.append(self.format(data, 'outTemp', 1))                # 3
+        fields.append(self.format(data, 'outHumidity', 0))            # 4
+        fields.append(self.format(data, 'dewpoint', 1))               # 5
+        fields.append(self.format(data, 'windSpeed_avg', 1))          # 6  *
+        fields.append(self.format(data, 'windSpeed', 1))              # 7
+        fields.append(self.format(data, 'windDir', 0))                # 8
+        fields.append(self.format(data, 'rainRate', 1))               # 9
+        fields.append(self.format(data, 'dayRain', 1))                # 10
+        fields.append(self.format(data, 'barometer', 1))              # 11
+        fields.append(self.format(data, 'windDir_compass'))           # 12 *
+        fields.append(self.format(data, 'windSpeed_beaufort'))        # 13 *
+        fields.append(self.format(data, 'units_wind'))                # 14 *
+        fields.append(self.format(data, 'units_temperature'))         # 15 *
+        fields.append(self.format(data, 'units_pressure'))            # 16 *
+        fields.append(self.format(data, 'units_rain'))                # 17 *
+        fields.append(self.format(data, 'wind_run', 1))               # 18 *
+        fields.append(self.format(data, 'pressure_trend', 1))         # 19 *
+        fields.append(self.format(data, 'rain_month', 1))             # 20 *
+        fields.append(self.format(data, 'rain_year', 1))              # 21 *
+        fields.append(self.format(data, 'rain_yesterday', 1))         # 22 *
+        fields.append(self.format(data, 'inTemp', 1))                 # 23
+        fields.append(self.format(data, 'inHumidity', 0))             # 24
+        fields.append(self.format(data, 'windchill', 1))              # 25
+        fields.append(self.format(data, 'temperature_trend', 1))      # 26 *
+        fields.append(self.format(data, 'outTemp_max', 1))            # 27 *
+        fields.append(self.format(data, 'outTemp_max_time'))          # 28 *
+        fields.append(self.format(data, 'outTemp_min', 1))            # 29 *
+        fields.append(self.format(data, 'outTemp_min_time'))          # 30 *
+        fields.append(self.format(data, 'windSpeed_max', 1))          # 31 *
+        fields.append(self.format(data, 'windSpeed_max_time'))        # 32 *
+        fields.append(self.format(data, 'windGust_max', 1))           # 33 *
+        fields.append(self.format(data, 'windGust_max_time'))         # 34 *
+        fields.append(self.format(data, 'pressure_max', 1))           # 35 *
+        fields.append(self.format(data, 'pressure_max_time'))         # 36 *
+        fields.append(self.format(data, 'pressure_min', 1))           # 37 *
+        fields.append(self.format(data, 'pressure_min_time'))         # 38 *
+        fields.append('%s' % weewx.__version__)                       # 39
+        fields.append('0')                                            # 40
+        fields.append(self.format(data, '10min_high_gust', 1))        # 41 *
+        fields.append(self.format(data, 'heatindex', 1))              # 42 *
+        fields.append(self.format(data, 'humidex', 1))                # 43 *
+        fields.append(self.format(data, 'UV', 0))                     # 44
+        fields.append(self.format(data, 'ET_today', 1))               # 45 *
+        fields.append(self.format(data, 'radiation'))                 # 46
+        fields.append(self.format(data, '10min_avg_wind_bearing', 0)) # 47 *
+        fields.append(self.format(data, 'rain_hour'))                 # 48 *
+        fields.append('0')                                            # 49
+        fields.append(self.format(data, 'is_daylight'))               # 50 *
+        fields.append(self.format(data, 'lost_sensors_contact'))      # 51 *
+        fields.append(self.format(data, 'avg_wind_dir'))              # 52 *
+        fields.append(self.format(data, 'cloud_base', 0))             # 53 *
+        fields.append(self.format(data, 'units_cloud_base'))          # 54 *
+        fields.append(self.format(data, 'apparent_temperature', 1))   # 55 *
+        fields.append(self.format(data, 'sunshine_hours', 1))         # 56 *
+        fields.append(self.format(data, 'max_rad', 1))                # 57 ##
+        fields.append(self.format(data, 'is_sunny'))                  # 58 *
         return ' '.join(fields)
